@@ -43,11 +43,8 @@
 (require 'alist)
 (require 'path-util)
 
-(defsubst mime-image-normalize-xbm-buffer (buffer)
-  ;; Do not use `with-current-buffer'.
-  ;; buffer may be the current buffer.
+(defsubst mime-image-normalize-xbm-buffer ()
   (save-excursion
-    (set-buffer buffer)
     (let ((case-fold-search t) width height xbytes right margin)
       (goto-char (point-min))
       (or (re-search-forward "_width[\t ]+\\([0-9]+\\)" nil t)
@@ -87,7 +84,7 @@
       (with-temp-buffer
 	(insert file-or-data)
 	(setq file-or-data
-	      (mime-image-normalize-xbm-buffer (current-buffer)))))
+	      (mime-image-normalize-xbm-buffer))))
     (let ((glyph
 	   (make-glyph
 	    (if (and type (mime-image-type-available-p type))
@@ -108,53 +105,72 @@
  ((require 'image nil t)
   (defcustom mime-image-max-height nil
     "*Max displayed image height of attachment image to a message.
-It has effect only when imagemagick support is available."
+It has effect only when imagemagick or image scaling support is
+available.
+When `mime-image-normalize-xbm' is non-nil, original size is
+always used for xbm image."
     :group 'mime-view
     :type '(choice (const :tag "Use original size" nil)
 		   integer))
 
   (defcustom mime-image-max-width nil
     "*Max displayed image width of attachment image to a message.
-It has effect only when imagemagick support is available."
+It has effect only when imagemagick or image scaling support is
+available.
+When `mime-image-normalize-xbm' is non-nil, original size is
+always used for xbm image."
     :group 'mime-view
     :type '(choice (const :tag "Use original size" nil)
 		   integer))
 
+  (defcustom mime-image-normalize-xbm t
+    "*When non-nil, build binary xbm image to display.
+Furthermore, image scaling for xbm image is disabled."
+    :group 'mime-view
+    :type 'boolean)
+
   (defalias 'mime-image-type-available-p 'image-type-available-p)
   (defun mime-image-create
       (file-or-data &optional type data-p &rest props)
-    (let ((imagemagick
-	   (and (image-type-available-p 'imagemagick)
-		(fboundp 'imagemagick-filter-types)
-		(member (downcase (symbol-name type))
-			(mapcar (lambda (e) (downcase (symbol-name e)))
-				(imagemagick-filter-types)))
-		(or mime-image-max-height mime-image-max-width))))
-      (cond
-       (imagemagick
-	(apply #'create-image file-or-data 'imagemagick data-p
-	       (nconc (and mime-image-max-width
-			   `(:max-width ,mime-image-max-width))
-		      (and mime-image-max-height
-			   `(:max-heigh ,mime-image-max-height))
-		      props)))
-       ((and data-p (eq type 'xbm))
+    (let* ((scale-p (and (fboundp 'image-transforms-p)
+			 (image-transforms-p)))
+	   (imagemagick
+	    (and (null scale-p)
+		 (or mime-image-max-height mime-image-max-width)
+		 (image-type-available-p 'imagemagick)
+		 (fboundp 'imagemagick-filter-types)
+		 (member (downcase (symbol-name type))
+			 (mapcar (lambda (e) (downcase (symbol-name e)))
+				 (imagemagick-filter-types)))))
+	   height width)
+      (when (and mime-image-normalize-xbm data-p (eq type 'xbm))
 	(with-temp-buffer
 	  (insert file-or-data)
 	  (setq file-or-data
-		(mime-image-normalize-xbm-buffer (current-buffer)))
-	  (apply #'create-image (nth 2 file-or-data) type data-p
-		 (nconc
-		  (list :width (car file-or-data)
-			:height (nth 1 file-or-data))
-		  props))))
+		(mime-image-normalize-xbm-buffer)))
+	(setq width (car file-or-data)
+	      height (nth 1 file-or-data)
+	      file-or-data (nth 2 file-or-data)))
+      (setq props
+	    (nconc (and width `(:width ,width))
+		   (and height `(:height ,height))
+		   (and (or scale-p imagemagick)
+			mime-image-max-width scale-p
+			`(:max-width ,mime-image-max-width))
+		   (and (or scale-p imagemagick)
+			mime-image-max-height
+			`(:max-heigh ,mime-image-max-height))
+		   props))
+      (cond
+       (imagemagick
+	(apply #'create-image file-or-data 'imagemagick data-p props))
        (t
 	(apply #'create-image file-or-data type data-p props)))))
   (defalias 'mime-image-insert 'insert-image))
  (t
   (if (and (featurep 'mule) (require 'bitmap nil t))
       (progn
-	(defun mime-image-read-xbm-buffer (buffer)
+	(defun mime-image-read-xbm-buffer
 	  (condition-case nil
 	      (mapconcat #'bitmap-compose
 			 (append (bitmap-decode-xbm
@@ -189,7 +205,7 @@ It has effect only when imagemagick support is available."
 	(if data-p
 	    (insert file-or-data)
 	  (insert-file-contents file-or-data))
-	(mime-image-read-xbm-buffer (current-buffer)))))))
+	(mime-image-read-xbm-buffer))))))
 
 (defvar mime-image-format-alist
   '((image jpeg		jpeg)
