@@ -1,6 +1,7 @@
 ;;; mime-play.el --- Playback processing module for mime-view.el
 
-;; Copyright (C) 1994,95,96,97,98,99,2000 Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2002, 2003,
+;;   2004, 2010 Free Software Foundation, Inc.
 
 ;; Author: MORIOKA Tomohiko <tomo@m17n.org>
 ;; Created: 1995/9/26 (separated from tm-view.el)
@@ -21,8 +22,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Code:
 
@@ -31,10 +32,8 @@
 (require 'filename)
 
 (eval-when-compile
-  (condition-case nil
-      (require 'bbdb)
-    (error (defvar bbdb-buffer-name nil)))
-  )
+  (unless (require 'bbdb nil t)
+    (defvar bbdb-buffer-name nil)))
 
 (defcustom mime-save-directory "~/"
   "*Name of the directory where MIME entity will be saved in.
@@ -157,7 +156,8 @@ specified, play as it.  Default MODE is \"play\"."
 		  (mime-format-mailcap-command
 		   method
 		   (cons (cons 'filename name) situation)))
-		 (coding-system-for-read mime-play-messages-coding-system))
+		 (coding-system-for-read mime-play-messages-coding-system)
+		 process-connection-type)
 	     (start-process command mime-echo-buffer-name
 	      shell-file-name shell-command-switch command))))
       (set-alist 'mime-mailcap-method-filename-alist process name)
@@ -280,7 +280,7 @@ window.")
 	(or (yes-or-no-p (format "File %s exists. Save anyway? " filename))
 	    (error "")))
     (mime-write-entity-content entity (expand-file-name filename))
-    ))
+    filename))
 
 
 ;;; @ file detection
@@ -332,8 +332,7 @@ SUBTYPE is symbol to indicate subtype of media-type.")
 It is registered to variable `mime-preview-quitting-method-alist'."
   (let ((mother mime-mother-buffer)
 	(win-conf mime-preview-original-window-configuration))
-    (if (and (boundp 'mime-view-temp-message-buffer)
-	     (buffer-live-p mime-view-temp-message-buffer))
+    (if (buffer-live-p mime-view-temp-message-buffer)
 	(kill-buffer mime-view-temp-message-buffer))
     (mime-preview-kill-buffer)
     (set-window-configuration win-conf)
@@ -378,16 +377,18 @@ occurs."
 	    (make-directory dir))
 	(set-default-file-modes orig-modes)))))
 
+(defvar mime-view-temp-message-buffer nil) ; buffer local variable
+
 (defun mime-store-message/partial-piece (entity cal)
-  (let* ((root-dir
-	  (expand-file-name
-	   (concat "m-prts-" (user-login-name)) temporary-file-directory))
-	 (id (cdr (assoc "id" cal)))
-	 (number (cdr (assoc "number" cal)))
-	 (total (cdr (assoc "total" cal)))
-	 file
-	 (mother (current-buffer))
-	 (orig-modes (default-file-modes)))
+  (let ((root-dir
+	 (expand-file-name
+	  (concat "m-prts-" (user-login-name)) temporary-file-directory))
+	(id (cdr (assoc "id" cal)))
+	(number (cdr (assoc "number" cal)))
+	(total (cdr (assoc "total" cal)))
+	file
+	(mother (current-buffer))
+	(orig-modes (default-file-modes)))
     (mime-require-safe-directory root-dir)
     (or (file-exists-p root-dir)
 	(unwind-protect
@@ -429,9 +430,8 @@ occurs."
 	      (if total
 		  (progn
 		    (or (file-exists-p total-file)
-			(save-excursion
-			  (set-buffer
-			   (get-buffer-create mime-temp-buffer-name))
+			(with-current-buffer
+			    (get-buffer-create mime-temp-buffer-name)
 			  (erase-buffer)
 			  (insert total)
 			  (write-region (point-min)(point-max) total-file)
@@ -440,8 +440,7 @@ occurs."
 		    (string-to-number total)
 		    )
 		(and (file-exists-p total-file)
-		     (save-excursion
-		       (set-buffer (find-file-noselect total-file))
+		     (with-current-buffer (find-file-noselect total-file)
 		       (prog1
 			   (and (re-search-forward "[0-9]+" nil t)
 				(string-to-number
@@ -455,8 +454,7 @@ occurs."
 	       (>= (length (directory-files root-dir nil "^[0-9]+$" t))
 		   total))
 	  (catch 'tag
-	    (save-excursion
-	      (set-buffer (get-buffer-create mime-temp-buffer-name))
+	    (with-current-buffer (get-buffer-create mime-temp-buffer-name)
 	      (let ((full-buf (current-buffer)))
 		(erase-buffer)
 		(let ((i 1))
