@@ -605,8 +605,9 @@
 
 (defun mime-tnef-parse (string)
   (catch :done
-    (unless (equal (string-make-unibyte (substring string 0 4))
-		   (string-make-unibyte "\x78\x9f\x3e\x22"))
+    (unless (and (> (length string) 6)
+		 (equal (string-make-unibyte (substring string 0 4))
+			(string-make-unibyte "\x78\x9f\x3e\x22")))
       (message "Input data does not seem to be MS-TNEF format")
       (throw :done nil))
     (mime-tnef-debug "TNEF Key: %04x\n" (mime-tnef-2bytes string 4))
@@ -664,11 +665,11 @@
     (decode-coding-string string
 			  (if (coding-system-p coding)
 			      coding
-			    (car (detect-coding-string string))))))
+			    (detect-coding-string string t)))))
 
 (defun mime-tnef-decode-unicode-string (string)
   (decode-coding-string string (and mime-tnef-unicode-coding-system
-				    (car (detect-coding-string string)))))
+				    (detect-coding-string string t))))
 
 
 ;;; MAPI
@@ -811,6 +812,28 @@
 
 ;;; MIME-view
 
+(defun mime-tnef-insert-file-parameters (params file data)
+  (let (charset
+	result)
+    (dolist (elt params)
+      (setq result
+	    (cons
+	     (cons
+	      (car elt)
+	      (if (eq (cdr elt) 'charset)
+		  (or charset
+		      (let ((codings (detect-coding-string data)))
+			(while (and (null charset) codings)
+			  (setq charset (coding-system-to-mime-charset
+					 (car codings)))
+			  (when charset
+			    (setq charset (symbol-name charset)))
+			  (setq codings (cdr codings)))
+			charset))
+		(cdr elt)))
+	     result)))
+    (mime-edit-insert-file-parameters (nreverse result) file)))
+
 (defun mime-tnef-insert-file (file data)
   (let*  ((guess (mime-find-file-type file))
 	  (type (nth 0 guess))
@@ -822,7 +845,7 @@
     (setq parameters
 	  (concat
 	   (when (consp parameters)
-	     (mime-edit-insert-file-parameters parameters file))
+	     (mime-tnef-insert-file-parameters parameters file data))
 	   (when disposition-type
 	     (concat "\n" "Content-Disposition: " disposition-type
 		     (mime-edit-insert-file-parameters
