@@ -80,12 +80,12 @@ buttom. Nil means don't scroll at all."
   :group 'mime-view
   :type 'boolean)
 
-(defcustom mime-view-multipart/related-show-all-children nil
+(defcustom mime-view-multipart/related-show-all-children t
   "When non-nil, do not hide child entities."
   :group 'mime-view
   :type 'boolean)
 
-(defcustom mime-view-multipart/alternative-show-all-children nil
+(defcustom mime-view-multipart/alternative-show-all-children t
   "When non-nil, show hidden descendant entities's buttons in
 multipart/alternative entities."
   :group 'mime-view
@@ -97,8 +97,10 @@ multipart/alternative entities."
 (defcustom mime-display-multipart/multilingual-prefered-languages
   (mapcar (lambda (lang)
 	    (format "^%s\\(-.+\\)?" (regexp-quote (symbol-name lang))))
-	  (let ((result (get-language-info
-			 current-language-environment 'iso639-language)))
+	  (let ((result (if (fboundp 'get-language-info)
+			    (get-language-info
+			     current-language-environment 'iso639-language)
+			  'en)))
 	    (if (eq result 'en)
 		(list result)
 	      (cons result '(en)))))
